--
-- extraWeights
--
-- @author:		madabub
-- @date:		08/08/17
-- @version:	v1.0
-- @history:	v1.0 - initial version
--
-- free for non-commercial usage

extraWeights = {};

function extraWeights.prerequisitesPresent(specializations)
	return true;
end;

function extraWeights:load(savegame)
	self.changeWeight = SpecializationUtil.callSpecializationsFunction("changeWeight");
	self.changeAttacher = SpecializationUtil.callSpecializationsFunction("changeAttacher");
	self.extraWeightsTriggerCallback = extraWeights.extraWeightsTriggerCallback;

	self.extraWeightsActiveWeights = 0;
	self.extraWeightsIsAttacherActive = false;
	
	self.extraWeights = {};
	self.extraWeights.extraWeightsActive = true;
	self.extraWeights.massComp = Utils.getNoNil(getXMLInt(self.xmlFile, 'vehicle.extraWeights#massComp'), 1);
	self.extraWeights.triggerIndex = Utils.indexToObject(self.components, getXMLString(self.xmlFile, 'vehicle.extraWeights#triggerIndex'));
	self.extraWeights.playerInTrigger = false;
	addTrigger(self.extraWeights.triggerIndex, "extraWeightsTriggerCallback", self);
	self.extraWeights.sendNumBits = Utils.getNoNil(getXMLInt(self.xmlFile, 'vehicle.extraWeights#sendNumBits'), 5);
	
	local attacherKey = 'vehicle.extraWeights.attacher';
	if hasXMLProperty(self.xmlFile, attacherKey) then
		self.extraWeights.hasAttacher = true;
		self.extraWeights.attacherMinWeights = Utils.getNoNil(getXMLInt(self.xmlFile, attacherKey .. '#minWeights'), 0);
		self.extraWeights.attacherChangeObjects = {};
		ObjectChangeUtil.loadObjectChangeFromXML(self.xmlFile, attacherKey, self.extraWeights.attacherChangeObjects, self.components, self);
		ObjectChangeUtil.setObjectChanges(self.extraWeights.attacherChangeObjects, false);
	end;
	
	self.extraWeights.weights = {}
	local i = 0;
	while true do
		local weightkey = string.format('vehicle.extraWeights.extraWeight(%d)', i);
		if not hasXMLProperty(self.xmlFile, weightkey) then
			break;
		end
		local extraWeight = {};
		extraWeight.mass = Utils.getNoNil(getXMLFloat(self.xmlFile, weightkey .. '#mass'), 0);
		extraWeight.changeObjects = {};
		ObjectChangeUtil.loadObjectChangeFromXML(self.xmlFile, weightkey, extraWeight.changeObjects, self.components, self);
		ObjectChangeUtil.setObjectChanges(extraWeight.changeObjects, false);
		table.insert(self.extraWeights.weights, extraWeight);
		i = i + 1;
	end;
	self.extraWeights.maxWeights = i;
end;

function extraWeights:postLoad(savegame)
	if self.configurations["attacherJoint"] ~= nil then
		local attacherConfigs = string.format("vehicle.attacherJointConfigurations.attacherJointConfiguration(%d)", self.configurations["attacherJoint"]-1);
		self.extraWeights.extraWeightsActive = Utils.getNoNil(getXMLBool(self.xmlFile, attacherConfigs .. '#extraWeights'), false);
	end;
	if savegame ~= nil then
		if self.extraWeights.extraWeightsActive then
			local key = savegame.key .. '.extraWeights';
			local activeWeights = getXMLInt(savegame.xmlFile, key .. '#activeWeights');
			self.extraWeightsMassBackup = getXMLFloat(savegame.xmlFile, key .. '#massBackup');
			if activeWeights ~= nil and self.extraWeightsMassBackup ~= nil then
				self:changeWeight(activeWeights, true);
			end;
			if self.extraWeights.hasAttacher then
				local isAttacherActive = getXMLBool(savegame.xmlFile, key .. '#isAttacherActive');
				if isAttacherActive ~= nil then
					self:changeAttacher(isAttacherActive, true);
				end;
			end;
		end;
	end;
end;

function extraWeights:delete()
	removeTrigger(self.extraWeights.triggerIndex);
end;

function extraWeights:getSaveAttributesAndNodes(nodeIdent)
	local nodes = nodeIdent .. '<extraWeights activeWeights="' .. self.extraWeightsActiveWeights .. '" massBackup="' .. self.extraWeightsMassBackup .. '"';
	if self.extraWeights.hasAttacher then
		nodes = nodes .. ' isAttacherActive="' .. tostring(self.extraWeightsIsAttacherActive) .. '"';
	end;
	nodes = nodes .. ' />';
	return '', nodes;
end;

function extraWeights:readStream(streamId, connection)
	self.extraWeightsMassBackup = streamReadFloat32(streamId);
	local activeWeights = streamReadUIntN(streamId, self.extraWeights.sendNumBits);
	self:changeWeight(activeWeights, true);
	if self.extraWeights.hasAttacher then
		local isAttacherActive = streamReadBool(streamId);
		self:changeAttacher(isAttacherActive, true);
	end;
end;

function extraWeights:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.extraWeightsMassBackup);
	streamWriteUIntN(streamId, self.extraWeightsActiveWeights, self.extraWeights.sendNumBits);
	if self.extraWeights.hasAttacher then
		streamWriteBool(streamId, self.extraWeightsIsAttacherActive);
	end;
end;

function extraWeights:mouseEvent(posX, posY, isDown, isUp, button)
end;

function extraWeights:keyEvent(unicode, sym, modifier, isDown)
end;

function extraWeights:updateTick(dt)
	if self.isServer then
		if self.extraWeightsMassBackup == nil then
			local mass = getMass(self.components[self.extraWeights.massComp].node);
			local i = 0;
			if mass ~= 1 then
				self.extraWeightsMassBackup = mass;
			elseif i >= 3 then
				self.extraWeightsMassBackup = mass;
			end;
			i = i + 1;
		end;
	end;
end;

function extraWeights:update(dt)
	if self.isClient then
		if self.extraWeights.playerInTrigger and g_currentMission.player.isEntered and self.extraWeightsMassBackup ~= nil then
			if self.extraWeightsActiveWeights >= 0 and self.extraWeightsActiveWeights < self.extraWeights.maxWeights then
				g_currentMission:addHelpButtonText(g_i18n:getText("extraWeights_addWeight"), InputBinding.IMPLEMENT_EXTRA2);
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
					self:changeWeight(self.extraWeightsActiveWeights + 1);
				end;
			end;
			if self.extraWeightsActiveWeights > 0 and self.extraWeightsActiveWeights <= self.extraWeights.maxWeights then
				g_currentMission:addHelpButtonText(g_i18n:getText("extraWeights_removeWeight"), InputBinding.IMPLEMENT_EXTRA4);
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
					self:changeWeight(self.extraWeightsActiveWeights - 1);
				end;
			end;
			if self.extraWeights.hasAttacher then
				if self.extraWeightsActiveWeights >= self.extraWeights.attacherMinWeights and self.extraWeightsActiveWeights <= self.extraWeights.maxWeights then
					if self.extraWeightsIsAttacherActive then
						g_currentMission:addHelpButtonText(g_i18n:getText("extraWeights_removeAttacher"), InputBinding.IMPLEMENT_EXTRA3);
					else
						g_currentMission:addHelpButtonText(g_i18n:getText("extraWeights_addAttacher"), InputBinding.IMPLEMENT_EXTRA3);
					end;
					if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA3) then
						self:changeAttacher(not self.extraWeightsIsAttacherActive);
					end;
				end;
			end;
			g_currentMission:addExtraPrintText(string.format(g_i18n:getText("extraWeights_currentWeight"), math.floor((getMass(self.components[self.extraWeights.massComp].node) - self.extraWeightsMassBackup)*1000+0.5)));
		end;
	end;
end;

function extraWeights:draw()
	if self.getIsActiveForInput then
		if self.extraWeights.extraWeightsActive then
			g_currentMission:addExtraPrintText(g_i18n:getText('extraWeights_info'));
		end;
	end;
end;

function extraWeights:extraWeightsTriggerCallback(triggerIndex, otherId, onEnter, onLeave, onStay, otherShapeId)
	if g_currentMission.player ~= nil and otherId == g_currentMission.player.rootNode then
		if onEnter then
			self.extraWeights.playerInTrigger = true;
		elseif onLeave then
			self.extraWeights.playerInTrigger = false;
		end;
	end;
end;

function extraWeights:getAdditiveClientMass()
	local mass = 0;
	for i = 1, self.extraWeightsActiveWeights do
		mass = mass + self.extraWeights.weights[i].mass;
	end;
	return mass;
end;

function extraWeights:changeWeight(activeWeights, noEventSend)
	changeWeightEvent.sendEvent(self, activeWeights, noEventSend);
	self.extraWeightsActiveWeights = activeWeights;
	if self.extraWeights.hasAttacher and self.extraWeightsIsAttacherActive then
		if activeWeights < self.extraWeights.attacherMinWeights then
			self:changeAttacher(false);
		end;
	end;
	for i, extraWeight in pairs(self.extraWeights.weights) do
		ObjectChangeUtil.setObjectChanges(extraWeight.changeObjects, false);
		if i <= activeWeights then
			ObjectChangeUtil.setObjectChanges(extraWeight.changeObjects, true);
		end;
	end;
	setMass(self.components[self.extraWeights.massComp].node, self.extraWeightsMassBackup + extraWeights.getAdditiveClientMass(self));
end;

function extraWeights:changeAttacher(isAttacherActive, noEventSend)
	changeAttacherEvent.sendEvent(self, isAttacherActive, noEventSend);
	self.extraWeightsIsAttacherActive = isAttacherActive;
	ObjectChangeUtil.setObjectChanges(self.extraWeights.attacherChangeObjects, isAttacherActive);
end;

--
-- changeWeightEvent
--
-- @author:		madabub
-- @date:		08/08/17
-- @version:	v1.0
-- @history:	v1.0 - initial version
--
-- free for non-commercial usage

changeWeightEvent = {};
InitEventClass(changeWeightEvent, "changeWeightEvent");

function changeWeightEvent:emptyNew()
	local self = Event:new(Class(changeWeightEvent, Event));
	self.className = "changeWeightEvent";
	return self;
end;

function changeWeightEvent:new(vehicle, activeWeights)
	local self = changeWeightEvent:emptyNew();
	self.vehicle = vehicle;
	self.extraWeightsActiveWeights = activeWeights;
	return self;
end;

function changeWeightEvent:readStream(streamId, connection)
	self.vehicle = readNetworkNodeObject(streamId);
	self.extraWeightsActiveWeights = streamReadUIntN(streamId, self.vehicle.extraWeights.sendNumBits);
	self:run(connection);
end;

function changeWeightEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteUIntN(streamId, self.extraWeightsActiveWeights, self.vehicle.extraWeights.sendNumBits);
end;

function changeWeightEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(self, false, connection, self.vehicle);
	end;
	if self.vehicle ~= nil then
		self.vehicle:changeWeight(self.extraWeightsActiveWeights, true);
	end;
end;

function changeWeightEvent.sendEvent(vehicle, activeWeights, noEventSend)
	if vehicle.extraWeightsActiveWeights ~= activeWeights then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(changeWeightEvent:new(vehicle, activeWeights), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(changeWeightEvent:new(vehicle, activeWeights));
			end;
		end;
	end;
end;

--
-- changeAttacherEvent
--
-- @author:		madabub
-- @date:		11/08/17
-- @version:	v1.0
-- @history:	v1.0 - initial version
--
-- free for non-commercial usage

changeAttacherEvent = {};
InitEventClass(changeAttacherEvent, "changeAttacherEvent");

function changeAttacherEvent:emptyNew()
	local self = Event:new(Class(changeAttacherEvent, Event));
	self.className = "changeAttacherEvent";
	return self;
end;

function changeAttacherEvent:new(vehicle, isAttacherActive)
	local self = changeAttacherEvent:emptyNew();
	self.vehicle = vehicle;
	self.isAttacherActive = isAttacherActive;
	return self;
end;

function changeAttacherEvent:readStream(streamId, connection)
	self.vehicle = readNetworkNodeObject(streamId);
	self.isAttacherActive = streamReadBool(streamId);
	self:run(connection);
end;

function changeAttacherEvent:writeStream(streamId, connection)
	writeNetworkNodeObject(streamId, self.vehicle);
	streamWriteBool(streamId, self.isAttacherActive);
end;

function changeAttacherEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(self, false, connection, self.vehicle);
	end;
	if self.vehicle ~= nil then
		self.vehicle:changeAttacher(self.isAttacherActive, true);
	end;
end;

function changeAttacherEvent.sendEvent(vehicle, isAttacherActive, noEventSend)
	if vehicle.extraWeightsIsAttacherActive ~= isAttacherActive then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(changeAttacherEvent:new(vehicle, isAttacherActive), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(changeAttacherEvent:new(vehicle, isAttacherActive));
			end;
		end;
	end;
end;